"""
WebDebug handler

This class wraps the HTTP server response.


WebDebug is web development proxy server that captures and displays HTTP
requests and responses.
Copyright (C) 1999  Paul Clip.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""

### IMPORTS

from WDconst import *
import HTTPmessage

import string
import whrandom


### CONSTANTS


### Classes

class HTTPresponse(HTTPmessage.HTTPmessage):
	"""
	Encapsulates an HTTP response.
	Member variables:
		raw_responseline: raw first line of HTTP response
		responseline: raw_responseline minus trailing whitespace
		version: the HTTP version of the request
		status_code: the HTTP response status code (e.g. 200, 302, etc.)
		status_message: the HTTP response status message (e.g. "OK", "Moved temporarily", etc.)
		raw_headers: a mapping of the headers included in the request
		headers: a mapping of the headers included in the request (with names in lowercase)
		raw_contents: the raw content of the request, if any (typically found in POST requests)
	"""

	def __init__(self, input, ip):
		HTTPmessage.HTTPmessage.__init__(self, ip)
		self.raw_responseline = self.readLine(input)
		self.responseline = string.strip(self.raw_responseline)
		words = string.split(self.responseline, None, 2)
		if len(words) == 3:
			[version, status_code, status_message] = words
			if version[:5] != 'HTTP/':
				self.printError("Bad response version (%s)" % version)
				return
		else:
			self.printError("Bad response syntax (%s)" % self.responseline)
			return
		self.version, self.status_code, self.status_message = version, status_code, status_message
		self.raw_headers, self.headers = self.parseHeaders(input)
		self.raw_contents = self.readContents(input, 1)

	def getContentType(self):
		return self.headers.get('content-type')

	def getContentLength(self):
		return len(self.raw_contents)

	def getRawContents(self):
		return self.raw_contents

	def sendResponse(self, output):
		""" writes the HTTP response to output, if no content-length was supplied by the server, we add it """
		output.write("%s %s %s\r\n" % (self.version, self.status_code, self.status_message))
		for h in self.raw_headers.keys():
			output.write("%s: %s\r\n" % (h, self.raw_headers[h]))
		if self.headers.get('content-length') == None:
			output.write("%s: %s\r\n" % ('Content-Length', self.getContentLength()))
		output.write("\r\n")
		output.write(self.raw_contents)

	def toHTML(self):
		""" render the response to HTML """
		html = HTTPmessage.HTTPmessage.toHTML(self, self.raw_responseline)
		html = html + "<tr><td>Server IP address</td><td>%s</td>\n" % self.ip_address
		if self.status_code == '200' and startsWith(self.headers.get('content-type', ""), 'image'):
			html = html + '<tr><td>Image</td><td><img src="%sself.displayResponse(%s,%s,\'all\')"></td></tr>' % (WD_PREFIX, id(self), whrandom.random())
		elif self.getContentLength() > 0:
			html = html + '<tr><td colspan=2><a href="%sself.displayResponse(%s,%s,\'html\')" target=_window>Display HTML instructions</a></td></tr>' % (WD_PREFIX, id(self), whrandom.random())
		html = html + '<tr><td colspan=2><a href="%sself.displayResponse(%s,%s,\'all\')" target=_window>Display HTTP response</a></td></tr>' % (WD_PREFIX, id(self), whrandom.random())
		html = html + "</table>\n"
		return html
